@@ -39,20 +39,20 @@ The following methods should be considered public:
 
 =cut
 
+package Astro::Coord::ECI::Sun;
+
 use strict;
 use warnings;
 
-package Astro::Coord::ECI::Sun;
-
-our $VERSION = '0.007';
+our $VERSION = '0.008';
 
 use base qw{Astro::Coord::ECI};
 
 use Astro::Coord::ECI::Utils qw{:all};
 use Carp;
-## use Data::Dumper;
+use Params::Util 0.25 qw{_CLASSISA};
 use POSIX qw{floor strftime};
-use UNIVERSAL qw{isa};
+use Time::y2038;
 
 my %static = (
     id => 'Sun',
@@ -62,7 +62,7 @@ my %static = (
 
 my $weaken = eval {
     require Scalar::Util;
-    UNIVERSAL::can ('Scalar::Util', 'weaken');
+    Scalar::Util->can('weaken');
     };
 my $object;
 
@@ -91,20 +91,19 @@ otherwise.
 =cut
 
 sub new {
-my $class = shift;
-if ($Singleton && $weaken && UNIVERSAL::isa ($class, __PACKAGE__)) {
-    if ($object) {
-	$object->set (@_) if @_;
-	return $object;
+    my ($class, @args) = @_;
+    ref $class and $class = ref $class;
+    if ($Singleton && $weaken && _CLASSISA($class, __PACKAGE__)) {
+	if ($object) {
+	    $object->set (@args) if @args;
+	    return $object;
+	} else {
+	    my $self = $object = $class->SUPER::new (%static, @args);
+	    $weaken->($object);
+	    return $self;
 	}
-      else {
-	my $self = $object = $class->SUPER::new (%static, @_);
-	$weaken->($object);
-	return $self;
-	}
-    }
-  else {
-    $class->SUPER::new (%static, @_);
+    } else {
+	return $class->SUPER::new (%static, @args);
     }
 }
 
@@ -146,20 +145,20 @@ my @quarters = ('Spring equinox', 'Summer solstice', 'Fall equinox',
 	'Winter solstice');
 
 sub almanac {
-my $self = shift;
-my $location = shift;
-ref $location && UNIVERSAL::isa ($location, 'Astro::Coord::ECI') or
-    croak <<eod;
+    my $self = shift;
+    my $location = shift;
+    embodies ($location, 'Astro::Coord::ECI') or
+	croak <<eod;
 Error - The first argument of the almanac() method must be a member of
         the Astro::Coord::ECI class, or a subclass thereof.
 eod
 
-my $start = shift || $self->universal;
-my $end = shift || $start + 86400;
+    my $start = shift || $self->universal;
+    my $end = shift || $start + 86400;
 
-my @almanac;
+    my @almanac;
 
-foreach (
+    foreach (
 	[$location, next_elevation => [$self, 0, 1], 'horizon',
 		['Sunset', 'Sunrise']],
 	[$location, next_meridian => [$self], 'transit',
@@ -168,17 +167,50 @@ foreach (
 		'twilight', ['end twilight', 'begin twilight']],
 	[$self, next_quarter => [], 'quarter',
 		[@quarters]],
-	) {
-    my ($obj, $method, $arg, $event, $descr) = @$_;
-    $obj->universal ($start);
-    while (1) {
-	my ($time, $which) = $obj->$method (@$arg);
-	last if $time >= $end;
-	push @almanac, [$time, $event, $which, $descr->[$which]]
-	    if $descr->[$which];
+    ) {
+	my ($obj, $method, $arg, $event, $descr) = @$_;
+	$obj->universal ($start);
+	while (1) {
+	    my ($time, $which) = $obj->$method (@$arg);
+	    last if $time >= $end;
+	    push @almanac, [$time, $event, $which, $descr->[$which]]
+		if $descr->[$which];
 	}
     }
-return sort {$a->[0] <=> $b->[0]} @almanac;
+    return (sort {$a->[0] <=> $b->[0]} @almanac);
+}
+
+=item @almanac = $sun->almanac_hash($location, $start, $end);
+
+This convenience method wraps $sun->almanac(), but returns a list of
+hash references, sort of like Astro::Coord::ECI::TLE->pass()
+does. The hashes contain the following keys:
+
+  {almanac} => {
+    {event} => the event type;
+    {detail} => the event detail (typically 0 or 1);
+    {description} => the event description;
+  }
+  {body} => the original object ($sun);
+  {station} => the observing station;
+  {time} => the time the quarter occurred.
+
+The {time}, {event}, {detail}, and {description} keys correspond to
+elements 0 through 3 of the list returned by almanac().
+
+=cut
+
+sub almanac_hash {
+    return map {
+	body => $_[0],
+	station => $_[1],
+	time => $_->[0],
+	almanac => {
+	    event => $_->[1],
+	    detail => $_->[2],
+	    description => $_->[3],
+	}
+    }, almanac(@_);
 }
 
 
@@ -192,8 +224,8 @@ implemented.
 =cut
 
 sub ecliptic_longitude {
-my $self = shift;
-($self->ecliptic ())[1];
+    my $self = shift;
+    return ($self->ecliptic ())[1];
 }
 
 
@@ -205,13 +237,13 @@ the last time set.
 =cut
 
 sub geometric_longitude {
-my $self = shift;
-croak <<eod unless defined $self->{_sun_geometric_longitude};
+    my $self = shift;
+    croak <<eod unless defined $self->{_sun_geometric_longitude};
 Error - You must set the time of the Sun object before the geometric
         longitude can be returned.
 eod
 
-$self->{_sun_geometric_longitude};
+    return $self->{_sun_geometric_longitude};
 }
 
 
@@ -246,37 +278,37 @@ L<http://en.wikipedia.org/wiki/Limb_darkening>.
     my $mean_mag = -26.8;
 
     sub magnitude {
-    my ($self, $theta, $omega) = @_;
-    return $mean_mag unless defined $theta;
-    unless (defined $omega) {
-	my @eci = $self->eci ();
-	$omega = $self->get ('diameter') / 2 /
-	    sqrt (distsq (\@eci[0 .. 2], [0, 0, 0]));
+	my ($self, $theta, $omega) = @_;
+	return $mean_mag unless defined $theta;
+	unless (defined $omega) {
+	    my @eci = $self->eci ();
+	    $omega = $self->get ('diameter') / 2 /
+		sqrt (distsq (\@eci[0 .. 2], [0, 0, 0]));
 	}
-    unless (defined $central_mag) {
-	my $sum = 0;
-	my $quotient = 2;
-	foreach my $a (@limb_darkening) {
-	    $sum += $a / $quotient++;
+	unless (defined $central_mag) {
+	    my $sum = 0;
+	    my $quotient = 2;
+	    foreach my $a (@limb_darkening) {
+		$sum += $a / $quotient++;
 	    }
-	$central_mag = $mean_mag - intensity_to_magnitude (2 * $sum);
+	    $central_mag = $mean_mag - intensity_to_magnitude (2 * $sum);
 	}
-    my $intens = 0;
-    my $point;
-    if ($theta < $omega) {
-	my $costheta = cos ($theta);
-	my $cosomega = cos ($omega);
-	my $sinomega = sin ($omega);
-	my $cospsi = sqrt ($costheta * $costheta -
-		$cosomega * $cosomega) / $sinomega;
-	my $psiterm = 1;
-	foreach my $a (@limb_darkening) {
-	    $intens += $a * $psiterm;
-	    $psiterm *= $cospsi;
+	my $intens = 0;
+	my $point;
+	if ($theta < $omega) {
+	    my $costheta = cos ($theta);
+	    my $cosomega = cos ($omega);
+	    my $sinomega = sin ($omega);
+	    my $cospsi = sqrt ($costheta * $costheta -
+		    $cosomega * $cosomega) / $sinomega;
+	    my $psiterm = 1;
+	    foreach my $a (@limb_darkening) {
+		$intens += $a * $psiterm;
+		$psiterm *= $cospsi;
 	    }
-	$point = $central_mag + intensity_to_magnitude ($intens);
+	    $point = $central_mag + intensity_to_magnitude ($intens);
 	}
-    return wantarray ? ($point, $intens, $central_mag) : $point;
+	return wantarray ? ($point, $intens, $central_mag) : $point;
     }
 }	# End local symbol block.
 
@@ -305,32 +337,67 @@ minutes.
 use constant QUARTER_INC => 86400 * 85;	# 85 days.
 
 sub next_quarter {
-my $self = shift;
-my $quarter = (defined $_[0] ? shift :
-    floor ($self->ecliptic_longitude () / PIOVER2) + 1) % 4;
-my $begin;
-while (floor ($self->ecliptic_longitude () / PIOVER2) == $quarter) {
-    $begin = $self->dynamical;
-    $self->dynamical ($begin + QUARTER_INC);
+    my ($self, $quarter) = @_;
+    $quarter = (defined $quarter ? $quarter :
+	floor ($self->ecliptic_longitude () / PIOVER2) + 1) % 4;
+    my $begin;
+    while (floor ($self->ecliptic_longitude () / PIOVER2) == $quarter) {
+	$begin = $self->dynamical;
+	$self->dynamical ($begin + QUARTER_INC);
     }
-while (floor ($self->ecliptic_longitude () / PIOVER2) != $quarter) {
-    $begin = $self->dynamical;
-    $self->dynamical ($begin + QUARTER_INC);
+    while (floor ($self->ecliptic_longitude () / PIOVER2) != $quarter) {
+	$begin = $self->dynamical;
+	$self->dynamical ($begin + QUARTER_INC);
     }
-my $end = $self->dynamical ();
+    my $end = $self->dynamical ();
 
-while ($end - $begin > 1) {
-    my $mid = floor (($begin + $end) / 2);
-    my $qq = floor ($self->dynamical ($mid)->ecliptic_longitude () / PIOVER2);
-    ($begin, $end) = $qq == $quarter ?
-	($begin, $mid) : ($mid, $end);
+    while ($end - $begin > 1) {
+	my $mid = floor (($begin + $end) / 2);
+	my $qq = floor ($self->dynamical ($mid)->ecliptic_longitude () /
+	    PIOVER2);
+	($begin, $end) = $qq == $quarter ?
+	    ($begin, $mid) : ($mid, $end);
     }
 
-$self->dynamical ($end);
+    $self->dynamical ($end);
 
-wantarray ? ($self->universal, $quarter, $quarters[$quarter]) : $self->universal;
+    return wantarray ? (
+	$self->universal, $quarter, $quarters[$quarter]) : $self->universal;
 }
 
+=item $hash_reference = $moon->next_quarter_hash($want);
+
+This convenience method wraps $moon->next_quarter(), but returns the
+data in a hash reference, sort of like Astro::Coord::ECI::TLE->pass()
+does. The hash contains the following keys:
+
+  {body} => the original object ($moon);
+  {almanac} => {
+    {event} => 'quarter',
+    {detail} => the quarter number (0 through 3);
+    {description} => the quarter description;
+  }
+  {time} => the time the quarter occurred.
+
+The {time}, {detail}, and {description} keys correspond to elements 0
+through 2 of the list returned by next_quarter().
+
+=cut
+
+sub next_quarter_hash {
+    my ($self, @args) = @_;
+    my ($time, $quarter, $desc) = $self->next_quarter(@args);
+    my %hash = (
+	body => $self,
+	almanac => {
+	    event => 'quarter',
+	    detail => $quarter,
+	    description => $desc,
+	},
+	time => $time,
+    );
+    return wantarray ? %hash : \%hash;
+}
 
 =item $period = $sun->period ()
 
@@ -341,7 +408,7 @@ per Appendix I (pg 408) of Jean Meeus' "Astronomical Algorithms,"
 
 =cut
 
-sub period {31558149.7632}	# 365.256363 * 86400
+sub period {return 31558149.7632}	# 365.256363 * 86400
 
 
 =item $sun->time_set ()
@@ -376,29 +443,27 @@ use constant SUN_C3_0 => deg2rad (0.000289);
 use constant SUN_LON_2000 => deg2rad (- 0.01397);
 
 sub time_set {
-my $self = shift;
-
-my $time = $self->dynamical;
-
+    my $self = shift;
+    my $time = $self->dynamical;
 
 #	The following algorithm is from Meeus, chapter 25, page, 163 ff.
 
-my $T = jcent2000 ($time);				# Meeus (25.1)
-my $L0 = mod2pi (deg2rad ((.0003032 * $T + 36000.76983) * $T	# Meeus (25.2)
-	+ 280.46646));
-my $M = mod2pi (deg2rad (((-.0001537) * $T + 35999.05029)	# Meeus (25.3)
-	* $T + 357.52911));
-my $e = (-0.0000001267 * $T - 0.000042037) * $T + 0.016708634;	# Meeus (25.4)
-my $C  = ((SUN_C1_2 * $T + SUN_C1_1) * $T + SUN_C1_0) * sin ($M)
-	+ (SUN_C2_1 * $T + SUN_C2_0) * sin (2 * $M)
-	+ SUN_C3_0 * sin (3 * $M);
-my $O = $self->{_sun_geometric_longitude} = $L0 + $C;
-my $omega = mod2pi (deg2rad (125.04 - 1934.156 * $T));
-my $lambda = mod2pi ($O - deg2rad (0.00569 + 0.00478 * sin ($omega)));
-my $nu = $M + $C;
-my $R = (1.000_001_018 * (1 - $e * $e)) / (1 + $e * cos ($nu))
-	* AU;
-$self->{debug} and print <<eod;
+    my $T = jcent2000 ($time);				# Meeus (25.1)
+    my $L0 = mod2pi(deg2rad((.0003032 * $T + 36000.76983) * $T	# Meeus (25.2)
+	    + 280.46646));
+    my $M = mod2pi(deg2rad(((-.0001537) * $T + 35999.05029)	# Meeus (25.3)
+	    * $T + 357.52911));
+    my $e = (-0.0000001267 * $T - 0.000042037) * $T + 0.016708634;# Meeus (25.4)
+    my $C  = ((SUN_C1_2 * $T + SUN_C1_1) * $T + SUN_C1_0) * sin ($M)
+	    + (SUN_C2_1 * $T + SUN_C2_0) * sin (2 * $M)
+	    + SUN_C3_0 * sin (3 * $M);
+    my $O = $self->{_sun_geometric_longitude} = $L0 + $C;
+    my $omega = mod2pi (deg2rad (125.04 - 1934.156 * $T));
+    my $lambda = mod2pi ($O - deg2rad (0.00569 + 0.00478 * sin ($omega)));
+    my $nu = $M + $C;
+    my $R = (1.000_001_018 * (1 - $e * $e)) / (1 + $e * cos ($nu))
+	    * AU;
+    $self->{debug} and print <<eod;
 Debug sun - @{[strftime '%d-%b-%Y %H:%M:%S', gmtime $time]}
     T  = $T
     L0 = @{[_rad2deg ($L0)]} degrees
@@ -411,10 +476,10 @@ Debug sun - @{[strftime '%d-%b-%Y %H:%M:%S', gmtime $time]}
     lambda = @{[_rad2deg ($lambda)]} degrees
 eod
 
-$self->ecliptic (0, $lambda, $R);
-## $self->set (equinox_dynamical => $time);
-$self->equinox_dynamical ($time);
-$self;
+    $self->ecliptic (0, $lambda, $R);
+    ## $self->set (equinox_dynamical => $time);
+    $self->equinox_dynamical ($time);
+    return $self;
 }
 
 1;
@@ -445,7 +510,7 @@ Thomas R. Wyant, III (F<wyant at cpan dot org>)
 
 =head1 COPYRIGHT
 
-Copyright 2005, 2006, 2007, 2008 by Thomas R. Wyant, III
+Copyright 2005, 2006, 2007, 2008, 2009 by Thomas R. Wyant, III
 (F<wyant at cpan dot org>). All rights reserved.
 
 =head1 LICENSE
